package betterwithmods.module.hardcore.world;

import betterwithmods.common.blocks.BlockStump;
import betterwithmods.common.world.gen.feature.*;
import betterwithmods.module.Feature;
import net.minecraft.block.*;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.chunk.Chunk;
import net.minecraft.world.gen.feature.WorldGenerator;
import net.minecraftforge.event.terraingen.DecorateBiomeEvent;
import net.minecraftforge.event.terraingen.SaplingGrowTreeEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import javax.annotation.Nullable;
import java.util.Random;

/**
 * Created by tyler on 4/20/17.
 */
public class HCStumping extends Feature {
    /**
     * Leaves and logs are skipped to reach the raw terrain surface
     *
     * @param state State to check.
     */
    private static boolean shouldBeSkipped(IBlockState state) {
        return state.func_177230_c() == Blocks.field_150362_t ||
                state.func_177230_c() == Blocks.field_150361_u ||
                state.func_177230_c() == Blocks.field_150364_r ||
                state.func_177230_c() == Blocks.field_150363_s ||
                state.func_177230_c() == Blocks.field_150350_a;
    }

    /**
     * Whether a tree could be on top of the block.
     * Copy of {@link BlockBush#canSustainBush(IBlockState)} build 2185
     *
     * @param state State to check.
     */
    private static boolean canSustainTree(IBlockState state) {
        return state.func_177230_c() == Blocks.field_150349_c || state.func_177230_c() == Blocks.field_150346_d || state.func_177230_c() == Blocks.field_150458_ak;
    }

    /**
     * Hardcore Stumping
     * Checks various properties to find a {@link BlockPlanks.EnumType} for the state.
     *
     * @param state State of the wooden block to get wood type from.
     * @return The type if it has any, null otherwise.
     */
    @Nullable
    public static BlockPlanks.EnumType getWoodType(IBlockState state) {
        if (state.func_177228_b().containsKey(BlockPlanks.field_176383_a)) {
            return state.func_177229_b(BlockPlanks.field_176383_a);
        } else if (state.func_177228_b().containsKey(BlockOldLog.field_176301_b)) {
            return state.func_177229_b(BlockOldLog.field_176301_b);
        } else if (state.func_177228_b().containsKey(BlockNewLog.field_176300_b)) {
            return state.func_177229_b(BlockNewLog.field_176300_b);
        } else return null;
    }

    /**
     * Hardcore Stumping
     * <p>
     * Edited version of {@link BlockSapling#isTwoByTwoOfType(World, BlockPos, int, int, BlockPlanks.EnumType)} build 2185
     */
    private static boolean isTwoByTwoOfType(BlockSapling sapling, World worldIn, BlockPos pos, int p_181624_3_, int p_181624_4_, BlockPlanks.EnumType type) {
        return sapling.func_176477_a(worldIn, pos.func_177982_a(p_181624_3_, 0, p_181624_4_), type) && sapling.func_176477_a(worldIn, pos.func_177982_a(p_181624_3_ + 1, 0, p_181624_4_), type) && sapling.func_176477_a(worldIn, pos.func_177982_a(p_181624_3_, 0, p_181624_4_ + 1), type) && sapling.func_176477_a(worldIn, pos.func_177982_a(p_181624_3_ + 1, 0, p_181624_4_ + 1), type);
    }

    /**
     * Hardcore Stumping
     * <p>
     * Edited version of {@link BlockSapling#generateTree(World, BlockPos, IBlockState, Random)} build 2185
     */
    public static void generateTreeWithStump(World worldIn, BlockPos pos, IBlockState state, Random rand) {
        if (!(state.func_177230_c() instanceof BlockSapling)) return;
        BlockSapling sapling = (BlockSapling) state.func_177230_c();
        WorldGenerator worldgenerator = rand.nextInt(10) == 0 ? new WorldGenBigTreeWithStump(true) : new WorldGenTreesWithStump(true);
        int i = 0;
        int j = 0;
        boolean flag = false;

        switch (state.func_177229_b(BlockSapling.field_176480_a)) {
            case SPRUCE:
                label114:

                for (i = 0; i >= -1; --i) {
                    for (j = 0; j >= -1; --j) {
                        if (isTwoByTwoOfType(sapling, worldIn, pos, i, j, BlockPlanks.EnumType.SPRUCE)) {
                            worldgenerator = new WorldGenMegaPineTreeWithStump(false, rand.nextBoolean());
                            flag = true;
                            break label114;
                        }
                    }
                }

                if (!flag) {
                    i = 0;
                    j = 0;
                    worldgenerator = new WorldGenTaiga2WithStump(true);
                }

                break;
            case BIRCH:
                worldgenerator = new WorldGenBirchTreeWithStump(true, false);
                break;
            case JUNGLE:
                IBlockState iblockstate = Blocks.field_150364_r.func_176223_P().func_177226_a(BlockOldLog.field_176301_b, BlockPlanks.EnumType.JUNGLE);
                IBlockState iblockstate1 = Blocks.field_150362_t.func_176223_P().func_177226_a(BlockOldLeaf.field_176239_P, BlockPlanks.EnumType.JUNGLE).func_177226_a(BlockLeaves.field_176236_b, Boolean.FALSE);
                label269:

                for (i = 0; i >= -1; --i) {
                    for (j = 0; j >= -1; --j) {
                        if (isTwoByTwoOfType(sapling, worldIn, pos, i, j, BlockPlanks.EnumType.JUNGLE)) {
                            worldgenerator = new WorldGenMegaJungleWithStump(true, 10, 20, iblockstate, iblockstate1);
                            flag = true;
                            break label269;
                        }
                    }
                }

                if (!flag) {
                    i = 0;
                    j = 0;
                    worldgenerator = new WorldGenTreesWithStump(true, 4 + rand.nextInt(7), iblockstate, iblockstate1, false);
                }

                break;
            case ACACIA:
                worldgenerator = new WorldGenSavannaTreeWithStump(true);
                break;
            case DARK_OAK:
                label390:

                for (i = 0; i >= -1; --i) {
                    for (j = 0; j >= -1; --j) {
                        if (isTwoByTwoOfType(sapling, worldIn, pos, i, j, BlockPlanks.EnumType.DARK_OAK)) {
                            worldgenerator = new WorldGenCanopyTreeWithStump(true);
                            flag = true;
                            break label390;
                        }
                    }
                }

                if (!flag) {
                    return;
                }

            case OAK:
        }

        IBlockState iblockstate2 = Blocks.field_150350_a.func_176223_P();

        if (flag) {
            worldIn.func_180501_a(pos.func_177982_a(i, 0, j), iblockstate2, 4);
            worldIn.func_180501_a(pos.func_177982_a(i + 1, 0, j), iblockstate2, 4);
            worldIn.func_180501_a(pos.func_177982_a(i, 0, j + 1), iblockstate2, 4);
            worldIn.func_180501_a(pos.func_177982_a(i + 1, 0, j + 1), iblockstate2, 4);
        } else {
            worldIn.func_180501_a(pos, iblockstate2, 4);
        }

        if (!worldgenerator.func_180709_b(worldIn, rand, pos.func_177982_a(i, 0, j))) {
            if (flag) {
                worldIn.func_180501_a(pos.func_177982_a(i, 0, j), state, 4);
                worldIn.func_180501_a(pos.func_177982_a(i + 1, 0, j), state, 4);
                worldIn.func_180501_a(pos.func_177982_a(i, 0, j + 1), state, 4);
                worldIn.func_180501_a(pos.func_177982_a(i + 1, 0, j + 1), state, 4);
            } else {
                worldIn.func_180501_a(pos, state, 4);
            }
        }
    }

    @Override
    public String getFeatureDescription() {
        return "Makes the bottom block of trees into stumps which cannot be removed by hand, making your mark on the world more obvious";
    }

    @Override
    public void init(FMLInitializationEvent event) {
//        BWMRecipes.addShapelessOreRecipe(new ItemStack(BWMItems.STUMP_REMOVER, 2), new ItemStack(BWMItems.CREEPER_OYSTER), new ItemStack(Blocks.RED_MUSHROOM), new ItemStack(Items.ROTTEN_FLESH));
    }

    @Override
    public boolean requiresMinecraftRestartToEnable() {
        return true;
    }

    /**
     * Add stumps to trees added during world generation.
     *
     * @param event Triggered for each chunk cross-shaped intersection (hence the +8 offsets)
     */
    @SubscribeEvent
    public void addStumpsToGeneratedTrees(DecorateBiomeEvent.Post event) {
        if (!event.getWorld().field_73011_w.func_76569_d()) return;
        for (int dx = 0; dx < 16; dx++) {
            int x = event.getPos().func_177958_n() + dx;
            x += 8;
            for (int dz = 0; dz < 16; dz++) {
                int z = event.getPos().func_177952_p() + dz;
                z += 8;
                Chunk chunk = event.getWorld().func_175726_f(new BlockPos(x, 0, z));
                int y = chunk.func_76611_b(x & 15, z & 15) - 1;
                BlockPos pos = new BlockPos(x, y, z);
                IBlockState state = event.getWorld().func_180495_p(pos);
                while (shouldBeSkipped(state)) {
                    y--;
                    pos = new BlockPos(x, y, z);
                    state = event.getWorld().func_180495_p(pos);
                }
                if (BlockStump.canPlaceStump(event.getWorld(), pos.func_177984_a()) && canSustainTree(state)) {
                    IBlockState stump = BlockStump.getStump(event.getWorld().func_180495_p(pos.func_177984_a()));
                    if (stump != null) {
                        event.getWorld().func_175656_a(pos.func_177984_a(), stump);
                    }
                }
            }
        }
    }

    @SubscribeEvent
    public void addStumpToTree(SaplingGrowTreeEvent event) {
        IBlockState state = event.getWorld().func_180495_p(event.getPos());
        if (state.func_177230_c() != Blocks.field_150345_g)/*(!(state.getBlock() instanceof BlockSapling))*/ return;
        event.setResult(Event.Result.DENY);
        generateTreeWithStump(event.getWorld(), event.getPos(), state, event.getRand());
    }

    @Override
    public boolean hasSubscriptions() {
        return true;
    }
}
